﻿function Generate-KeyVaultUrl
{
    param([string]$resourceGroupName,
          [string]$keyVaultName,
          [string]$secretName,
          [string]$location,
          [string]$certificatePath,
          [string]$password)

    Create-AzureResourceGroupIfNotExist -resourceGroupName $resourceGroupName -location $location
    Create-AzureKeyVaultIfNotExist -azureKeyVaultName $keyVaultName -ResourceGroupName $resourceGroupName -Location $location
    $secret = Get-SecretValueForAzureKeyVault -certificatePath $certificatePath -certificatePassword $password
    $azureKeyVaultSecret = Set-AzureKeyVaultSecret -VaultName $keyVaultName -Name $secretName -SecretValue $secret
    Write-Verbose -Verbose "Created a secret in an Azure Key Vault"
    Write-Verbose -Verbose "Secret Id: $($azureKeyVaultSecret.Id)"
    return $azureKeyVaultSecret.Id
}

function Create-AzureResourceGroupIfNotExist
{
	param([string]$resourceGroupName,
	[string]$location)

	$azureResourceGroup = Get-AzureRmResourceGroup -Name $resourceGroupName -ErrorAction SilentlyContinue
	
	if(!$azureResourceGroup)
	{
		Write-Verbose -Verbose "Creating resource group $resourceGroupName in $location"

		$response = New-AzureRmResourceGroup -Name $resourceGroupName -Location $location -Verbose -ErrorAction Stop

		Write-Verbose -Verbose "Created resource group $resourceGroupName"
	}
}

function Get-SecretValueForAzureKeyVault
{
    param([string]$certificatePath,
          [string]$certificatePassword)

    $fileContentBytes = Get-Content $certificatePath -Encoding Byte
    $fileContentEncoded = [System.Convert]::ToBase64String($fileContentBytes)

    $jsonObject = "
    {
    ""data"": ""$filecontentencoded"",
    ""dataType"" :""pfx"",
    ""password"": ""$certificatePassword""
    }"

    $jsonObjectBytes = [System.Text.Encoding]::UTF8.GetBytes($jsonObject)
    $jsonEncoded = [System.Convert]::ToBase64String($jsonObjectBytes)

    $secret = ConvertTo-SecureString -String $jsonEncoded -AsPlainText –Force

    return $secret
}

function Create-AzureKeyVaultIfNotExist
{
	param([string]$azureKeyVaultName,
	[string]$resourceGroupName,
	[string]$location)

	$azureKeyVault = Get-AzureRmKeyVault -VaultName $azureKeyVaultName -ResourceGroupName $resourceGroupName -ErrorAction SilentlyContinue

	if($azureKeyVault -eq $null)
	{
		Write-Verbose -Verbose "Creating Azure Key Vault with name $azureKeyVaultName in group $resourceGroupName at $location"

		$response = New-AzureRmKeyVault -VaultName $azureKeyVaultName -ResourceGroupName $resourceGroupName -Location $location -EnabledForDeployment -ErrorAction Stop

		Write-Verbose -Verbose "Created Azure Key Vault for secrets"
	}
	else
	{
		if($azureKeyVault.EnabledForDeployment -eq $false)
		{
			throw "Secrets not enabled to be retrieved from KeyVault $azureKeyVaultName by the Microsoft.Compute resource provider, can't proceed with WinRM configuration"
		}
	}
}